"""The model module handles all logic/calculations, e.g. calculate statistics, testing for special conditions."""
from ydata_profiling.model.description import BaseAnalysis, BaseDescription

__all__ = ["BaseAnalysis", "BaseDescription"]
